@@ -21,9 +21,9 @@ import sys
 from packaging.specifiers import InvalidSpecifier, SpecifierSet
 from packaging.version import InvalidVersion, Version
 
-if sys.version_info >= (3, 8):  # pragma: no cover
-    import importlib.metadata as metadata
-else:  # pragma: no cover
+if sys.version_info >= (3, 8):
+    from importlib import metadata
+else:
     import importlib_metadata as metadata
 
 
@@ -37,7 +37,7 @@ class InvalidVersionSpecifier(Exception):
 
 def get_nox_version() -> str:
     """Return the version of the installed Nox package."""
-    return metadata.version("nox")  # type: ignore[no-untyped-call, no-any-return]
+    return metadata.version("nox")
 
 
 def _parse_string_constant(node: ast.AST) -> str | None:  # pragma: no cover
