@@ -12,8 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
-from typing import Any, Callable, Iterable, List
+from collections.abc import Callable, Iterable
+from typing import Any
 
 
 def execute(
@@ -40,15 +43,16 @@ def execute(
     """
     try:
         # Iterate over each task and run it.
-        return_value = None
+        return_value: Any = None
         for function_ in workflow:
             # Send the previous task's return value if there was one.
-            args: List[Any] = []
+            args: list[Any] = []
             if return_value is not None:
                 args.append(return_value)
-            return_value = function_(*args, global_config=global_config)
+            kwargs: dict[str, Any] = {"global_config": global_config}
+            return_value = function_(*args, **kwargs)
 
-            # If we got a integer value as a result, abort task processing
+            # If we got an integer value as a result, abort task processing
             # and return it.
             if isinstance(return_value, int):
                 return return_value
