@@ -12,10 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import nox
 
 # Deliberately giving these silly names so we know this is not confused
-# with the projects noxfile
+# with the projects Noxfile
 
 
 @nox.session
